/*
 *  Copyright (C) Chaperon. All rights reserved.                               
 *  ------------------------------------------------------------------------- 
 *  This software is published under the terms of the Apache Software License 
 *  version 1.1, a copy of which has been included  with this distribution in 
 *  the LICENSE file.                                                         
 */

package net.sourceforge.chaperon.grammar.token.definition;

import java.io.Serializable;

import net.sourceforge.chaperon.grammar.Grammar;
import net.sourceforge.chaperon.helpers.Decoder;

/**
 * This class represents a interval of characters. Like<br/>
 * [a-z]
 *
 * @author Stephan Michels
 * @version $Revision: 1.2 $ $Date: 2002/05/06 16:22:56 $
 */
public class CharacterInterval implements CharacterClassElement, Serializable, Cloneable
{

  private char _minimum = 'a';
  private char _maximum = 'z';

  public CharacterInterval()
  {
  }

  /**
   * Sets the minimum character
   *
   * @param minimum Minimum character
   */
  public void setMinimum(char minimum)
  {
    _minimum = minimum;
  }

  /**
   * Returns the minimum character
   *
   * @return Minimum character
   */
  public char getMinimum()
  {
    return _minimum;
  }

  /**
   * Sets the maximum character
   *
   * @param maximum Maximum character
   */
  public void setMaximum(char maximum)
  {
    _maximum = maximum;
  }

  /**
   * Returns the maximum character
   *
   * @return Maximum character
   */
  public char getMaximum()
  {
    return _maximum;
  }

  /**
   * Returns the to comparing characters
   *
   * @return To comparing characters
   */
  public String getCharacters()
  {
    int c1 = (int) _minimum;
    int c2 = (int) _maximum;
    StringBuffer buffer = new StringBuffer();

    for (int c = c1; c <= c2; c++)
      buffer.append((char) c);

    return buffer.toString();
  }

  /**
   * Convert the character interval to a string
   *
   * @return Content of the interval
   */
  public String toString()
  {
    StringBuffer buffer = new StringBuffer();

    buffer.append(Decoder.decode(String.valueOf(_minimum)));
    buffer.append("-");
    buffer.append(Decoder.decode(String.valueOf(_maximum)));
    return buffer.toString();
  }

  public Object clone() throws CloneNotSupportedException
  { 
    CharacterInterval clone = new CharacterInterval();

    clone.setMinimum(getMinimum());
    clone.setMaximum(getMaximum());
    
    return clone;
  }
}
